#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Perform Anaphora Resolution on Extracted People """


from typing import List
from collections import defaultdict
from lingpatlab.baseblock import BaseObject, Stopwatch


LAST_TOKEN_BLACKLIST = [
    'the'
]


class PeopleSequenceAnalysis(BaseObject):
    """ Perform Anaphora Resolution on Extracted People """

    def __init__(self):
        """ Change Log

        Created:
            19-Mar-2024
            craigtrim@gmail.com
            *   in pursuit of
                https://github.com/craigtrim/datapipe-apis/issues/61
        Updated:
            27-Mar-2024
            craigtrim@gmail.com
            *   rewrite core algorithm in pursuit of testing 
                https://github.com/craigtrim/datapipe-apis/issues/71
        """
        BaseObject.__init__(self, __name__)

    @staticmethod
    def _cleanse(person: str) -> str:
        person = person.strip()
        if person.endswith("'s"):
            return person[:-2]
        return person

    def remove_subsumed_spans(self,
                              d_people: dict) -> dict:
        """
        Remove subsumed spans from the given dictionary.

        Args:
            d_people (dict): Dictionary containing people and their names.

        Returns:
            dict: Dictionary with subsumed spans removed.
        """

        for person_1, names_1 in d_people.items():
            for person_2, names_2 in d_people.items():
                if person_1 == person_2:
                    continue

                for name_1 in names_1:
                    for name_2 in names_2:

                        if name_1 == name_2:
                            d_people[person_1] = [
                                name for name in names_1 if name != name_2
                            ]

                        elif name_1 in name_2:
                            d_people[person_1] = [
                                name for name in names_1 if name_1 not in name_2
                            ]

                        elif name_2 in name_1:
                            d_people[person_2] = [
                                name for name in names_2 if name_2 not in name_1
                            ]

        return {
            k: d_people[k]
            for k in d_people
            if d_people[k]
        }

    @staticmethod
    def aggregate_last_tokens(d_people: dict) -> dict:
        """

        These shouldall be considered the same person:
            "Navy Frank Knox", "Frank Knox", "Secretary Knox"

        """

        d_aggregate_keys = defaultdict(list)

        for person in d_people:

            tokens = person.split()
            if len(tokens) == 1:
                d_aggregate_keys[person] = []
                continue

            last_token = tokens[-1]
            if last_token.lower() in ['jr'] and len(tokens) >= 2:
                last_token = tokens[-2]

            if last_token.lower() in LAST_TOKEN_BLACKLIST:
                continue

            d_aggregate_keys[last_token].append(person)

        return dict(d_aggregate_keys)

    @staticmethod
    def index_by_last_token(unigrams: List[str],
                            ngrams: List[str]) -> dict:
        """
        Indexes ngrams by their last token.

        Args:
            unigrams (List[str]): A list of individual tokens.
            Sample Input:
                ['Yorktown', 'Yatsushiro', "Yamato's", ... ]

            ngrams (List[str]): A list of ngrams.
            Sample Input:
                ['William Halsey', 'Soc McMorris', 'Robert Sherrod', ... ]

        Returns:
            dict: A dictionary where the keys are ngrams and the values are sets of last tokens.
            Sample Output:
                {'William Halsey': {'Halsey'}, 'Soc McMorris': {'McMorris'}, ... }
        """
        d_people = defaultdict(set)

        for ngram in ngrams:
            last_token = ngram.split()[-1]

            if last_token in unigrams:
                d_people[ngram].add(last_token)
            else:
                d_people[ngram] = []

        return dict(d_people)

    @staticmethod
    def cleanse(d_normalized: dict) -> dict:
        d_clean = {}

        for k in d_normalized:
            d_clean[k] = [
                person.replace(' . ', '. ').strip()
                for person in d_normalized[k]
            ]

        return d_clean

    def process(self,
                exact_matches: List[str],
                fuzzy_matches: List[str]) -> List[str]:
        """
        Process the exact and fuzzy matches to perform anaphora resolution on extracted people.

        Args:
            exact_matches (List[str]): A list of exact matches.
            fuzzy_matches (List[str]): A list of fuzzy matches.

        Returns:
            List[str]: A list of resolved people names.
        """
        sw = Stopwatch()

        people = exact_matches + fuzzy_matches

        people = [
            person for person in people
            if len(person) > 1 and len(person.split()[-1]) > 1
        ]

        unigrams = [
            person for person in people
            if len(person.split()) == 1
        ]

        ngrams = [
            person for person in people
            if person not in unigrams
        ]

        # Index ngrams by their last token
        d_people = self.index_by_last_token(unigrams, ngrams)

        # Aggregate people with the same last token
        d_aggregate = self.aggregate_last_tokens(d_people)

        # Remove subsumed spans from the aggregated people
        d_normalized = self.remove_subsumed_spans(d_aggregate)

        d_normalized = self.cleanse(d_normalized)

        if self.isEnabledForDebug and len(d_normalized):
            self.logger.debug(
                f"Anaphora Resolution: {d_normalized} in ({str(sw)})")

        return d_normalized



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Extract People Patterns from Text """


from typing import List, Dict
from lingpatlab.utils.dto import Sentence, SpacyResult
from lingpatlab.baseblock import BaseObject, Stopwatch


PATTERNS_1: List[str] = [
    'ADJ NOUN',  # this works well assuming title case of bigram
    'NOUN NOUN',
    'NOUN NOUN NOUN',
    'NOUN NOUN NOUN NOUN',
    'NOUN NOUN NOUN NOUN NOUN',
    'NOUN NOUN NOUN NOUN NOUN NOUN',
    'NOUN NOUN NOUN NOUN NOUN NOUN NOUN',
    'PROPN PROPN NUM',
    'PROPN ADP PROPN PROPN',
    'NOUN NOUN NOUN NOUN PUNCT NOUN',

    # Battle/PROPN of/ADP Cape/PROPN Esperance/PROPN
    # reference case: tests/extract_topics_01_test.py
    'PROPN ADP PROPN PROPN',
]


PATTERNS_2: List[str] = [
]


class TopicSequenceExtractor(BaseObject):
    """ Extract Topic Patterns from Text """

    def __init__(self):
        """ Change Log

        Created:
            22-Mar-2024
            craigtrim@gmail.com
            *   based on 'person-sequence-extractor'
                https://github.com/craigtrim/datapipe-apis/issues/63
        """
        BaseObject.__init__(self, __name__)

    @staticmethod
    def _filter(matching_sequences: List[str]) -> List[str]:
        """
        Filters the given list of matching sequences and returns a new list
        containing only the sequences that meet the specified criteria.

        Args:
            matching_sequences (List[str]): A list of matching sequences to be filtered.

        Returns:
            List[str]: A new list containing only the sequences that meet the criteria.
        """
        normalized = []

        for matching_sequence in matching_sequences:

            # no valid pattern can start with a period ...
            if matching_sequence.strip().startswith('.'):
                continue

            tokens = [
                token.strip() for token in
                matching_sequence.split()
            ]

            def is_upper() -> bool:

                # all tokens must be uppercased
                for token in tokens:

                    # an exception is made for tokens in this list
                    if not token[0].isupper() and not token in [
                        '.', 'of'
                    ]:
                        return False

                return True

            if is_upper():
                normalized.append(matching_sequence)

        return normalized

    @staticmethod
    def _cleanse(matching_tokens: List[SpacyResult]) -> str:
        """
        Cleanses the matching tokens by removing unnecessary characters and spaces.

        Args:
            matching_tokens (List[SpacyResult]): A list of SpacyResult objects representing the matching tokens.

        Returns:
            str: The cleansed sequence of tokens.

        """
        sequence = ' '.join([
            token.text for token
            in matching_tokens
        ])

        while '  ' in sequence:
            sequence = sequence.replace('  ', ' ')

        sequence = sequence.replace("'s", "")

        return sequence.strip()

    @staticmethod
    def _is_person(matching_tokens: List[SpacyResult]) -> bool:
        """
        Check if the given sequence of tokens represents a person.

        Args:
            matching_tokens (List[dict]): A list of tokens to be checked.

        Returns:
            bool: True if the sequence represents a person, False otherwise.
        """
        ents = ' '.join([
            token.ent for token in matching_tokens
        ])

        # if at least two ent=PERSON tags exist in this matching sequence
        # this is likely a PERSON and should be discarded as a topical match
        # since the PERSON extractor will likely pick it up
        return 'PERSON PERSON' in ents

    def _extract_patterns(self,
                          sentence: Sentence,
                          patterns: List[str]) -> List[str]:
        """
        Extract sequences from the given data based on the provided pattern.

        Args:
            data (List[List[str]]): The input data in List[List[str]] format.
            pattern (str): The pattern to match.

        Returns:
            List[List[str]]: List of sequences that match the given pattern.
        """

        matching_sequences = []

        for pattern in patterns:

            pattern: List[str] = pattern.split()

            for i, _ in enumerate(sentence.tokens):

                if len(pattern) + i > sentence.size():
                    continue

                if sentence.tokens[i].pos in [
                    'DET',
                ]:
                    continue

                def has_match() -> bool:

                    for j in range(len(pattern)):

                        if pattern[j] == '*':
                            continue

                        elif pattern[j] == 'NOUN' and sentence.tokens[i + j].pos == 'PROPN':
                            continue

                        elif pattern[j] == 'PROPN' and sentence.tokens[i + j].pos == 'NOUN':
                            continue

                        elif pattern[j] == 'PUNCT' and sentence.tokens[i + j].text.strip() == '.':
                            continue

                        elif pattern[j] != sentence.tokens[i + j].pos:
                            return False

                    return True

                is_person = self._is_person(sentence.tokens[i:i+len(pattern)])

                if has_match() and not is_person:
                    matching_sequences.append(
                        self._cleanse(
                            sentence.tokens[i: i + len(pattern)]
                        )
                    )

        matching_sequences = self._filter(matching_sequences)

        return matching_sequences

    def process(self,
                sentence: Sentence) -> Dict[str, List[str]]:
        """
        Process the given sentence and extract people sequences using exact and fuzzy matching patterns.

        Args:
            sentence (Sentence): The input sentence to process.

        Returns:
            Dict[str, List[str]]: A dictionary containing the extracted people sequences.
                The keys are "exact" and "fuzzy", and the values are lists of strings representing the matches.
        """
        sw = Stopwatch()
        assert isinstance(sentence, Sentence)

        exact_matches: List[str] = self._extract_patterns(
            sentence=sentence, patterns=PATTERNS_1)

        fuzzy_matches: List[str] = self._extract_patterns(
            sentence=sentence, patterns=PATTERNS_2)

        result = {
            "exact": exact_matches,
            "fuzzy": fuzzy_matches
        }

        total_matches = len(exact_matches) + len(fuzzy_matches)

        if self.isEnabledForDebug and total_matches > 0:
            self.logger.debug(
                f"Extracted Topic Sequences: {total_matches} in ({str(sw)})")

        return result



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Extract Part-of-Speech (POS) Patterns from Text """


from lingpatlab.baseblock import BaseObject
from typing import List, Dict, Optional
from lingpatlab.utils.dto import Sentences
from lingpatlab.analyze.svc import ExtractTopicEntities


class ExtractTopics(BaseObject):
    """ Extract Part-of-Speech (POS) Patterns from Text """

    def __init__(self):
        """ Change Log

        Created:
            28-Feb-2024
            craigtrim@gmail.com
            *   refactored out of a script in pursuit of
                https://github.com/craigtrim/datapipe-apis/issues/44
        Updated:
            14-Mar-2024
            craigtrim@gmail.com
            *   optionally pass in patterns
                https://github.com/craigtrim/datapipe-apis/issues/57
        Updated:
            19-Mar-2024
            craigtrim@gmail.com
            *   use exclude-people as a parameter in pursuit of
                https://github.com/craigtrim/datapipe-apis/issues/61#issuecomment-2008099602
        Updated:
            20-Mar-2024
            craigtrim@gmail.com
            *   renamed from 'extract-phrases' in pursuit of
                https://github.com/craigtrim/datapipe-apis/issues/62
        Updated:
            22-Mar-2024
            craigtrim@gmail.com
            *   refactored and based on 'extract-people'
                https://github.com/craigtrim/datapipe-apis/issues/63
        """
        BaseObject.__init__(self, __name__)
        self._extract_topics = ExtractTopicEntities().process

    def process(self,
                sentences: Sentences) -> Optional[Dict[str, List[str]]]:
        return self._extract_topics(sentences=sentences)




#!/usr/bin/env python
# -*- coding: UTF-8 -*-

# pylint:disable=bad-whitespace
# pylint:disable=line-too-long
# pylint:disable=too-many-lines
# pylint:disable=invalid-name

# #########################################################
#
#       ************** !! WARNING !! ***************
#       ******* THIS FILE WAS AUTO-GENERATED *******
#       ********* DO NOT MODIFY THIS FILE **********
#
# #########################################################

wordnet_terms_u_provenance = {
    'action': ['router.py',
               'plac_core.py',
               'csv2py_orchestrator.py',
               'wordnet2py.py',
               'csv_list_writer.py',
               'common_utils.py'],
    'source': 'resources/data/csv/wordnet.csv',
    'time': '2022-05-17 17:00:36.234958'}

wordnet_terms_u = [
    'uakari',
    'uakaris',
    'ubermensch',
    'ubiety',
    'ubiquinone',
    'ubiquitous',
    'ubiquitousness',
    'ubiquity',
    'ubykh',
    'uca',
    'uda',
    'udder',
    'udmurt',
    'udometer',
    'ufa',
    'ufo',
    'uganda',
    'ugandan',
    'ugaritic',
    'ugli',
    'uglify',
    'ugliness',
    'ugly',
    'ugrian',
    'ugric',
    'uhf',
    'uhland',
    'uighur',
    'uigur',
    'uintathere',
    'uintatheriidae',
    'uintatherium',
    'uk',
    'ukase',
    'uke',
    'ukempt',
    'ukraine',
    'ukrainian',
    'ukranian',
    'ukrayina',
    'ukulele',
    'ukuleles',
    'ulaanbaatar',
    'ulalgia',
    'ulama',
    'ulanova',
    'ulatrophia',
    'ulcer',
    'ulcerate',
    'ulcerated',
    'ulcerating',
    'ulceration',
    'ulcerative',
    'ulcerous',
    'ulcers',
    'ulema',
    'ulemorrhagia',
    'ulex',
    'ulfila',
    'ulfilas',
    'ulitis',
    'ull',
    'ullage',
    'ullr',
    'ulmaceae',
    'ulmus',
    'ulna',
    'ulnar',
    'ulster',
    'ult',
    'ulterior',
    'ulteriority',
    'ultima',
    'ultimacy',
    'ultimate',
    'ultimately',
    'ultimateness',
    'ultimatum',
    'ultimo',
    'ultra',
    'ultracef',
    'ultracentrifugation',
    'ultracentrifuge',
    'ultraconservative',
    'ultramarine',
    'ultramicroscope',
    'ultramicroscopic',
    'ultramodern',
    'ultramontane',
    'ultramontanism',
    'ultranationalism',
    'ultranationalistic',
    'ultrasonic',
    'ultrasonography',
    'ultrasound',
    'ultrasuede',
    'ultraviolet',
    'ululate',
    'ululation',
    'ulva',
    'ulvaceae',
    'ulvales',
    'ulvophyceae',
    'ulysses',
    'uma',
    'umayyad',
    'umbel',
    'umbellales',
    'umbellar',
    'umbellate',
    'umbellifer',
    'umbelliferae',
    'umbelliferous',
    'umbelliform',
    'umbellularia',
    'umbels',
    'umber',
    'umbilical',
    'umbilicate',
    'umbilicus',
    'umbo',
    'umbra',
    'umbrage',
    'umbrageous',
    'umbrella',
    'umbrellalike',
    'umbrellas',
    'umbrellawort',
    'umbria',
    'umbrian',
    'umbrina',
    'umbundu',
    'umlaut',
    'umma',
    'ummah',
    'ump',
    'umpirage',
    'umpire',
    'umpteen',
    'umpteenth',
    'umptieth',
    'umteen',
    'umteenth',
    'un',
    'unabashed',
    'unabated',
    'unable',
    'unabomber',
    'unabridged',
    'unabused',
    'unaccented',
    'unacceptability',
    'unacceptable',
    'unacceptableness',
    'unaccepted',
    'unaccessible',
    'unaccommodating',
    'unaccompanied',
    'unaccountable',
    'unaccredited',
    'unaccustomed',
    'unachievable',
    'unacknowledged',
    'unacquainted',
    'unacquisitive',
    'unactable',
    'unadaptability',
    'unadaptable',
    'unadapted',
    'unaddicted',
    'unaddressed',
    'unadjustable',
    'unadjusted',
    'unadoptable',
    'unadorned',
    'unadulterated',
    'unadventurous',
    'unadvisable',
    'unadvised',
    'unaerated',
    'unaesthetic',
    'unaffected',
    'unaffectedness',
    'unaffecting',
    'unaffectionate',
    'unaffiliated',
    'unaffixed',
    'unafraid',
    'unaged',
    'unaggressive',
    'unagitated',
    'unai',
    'unaided',
    'unaired',
    'unairworthy',
    'unalarming',
    'unalert',
    'unalienable',
    'unalike',
    'unalloyed',
    'unalterability',
    'unalterable',
    'unaltered',
    'unambiguity',
    'unambiguous',
    'unambiguously',
    'unambitious',
    'unamended',
    'unanalyzable',
    'unanalyzed',
    'unanimated',
    'unanimity',
    'unanimous',
    'unannealed',
    'unannounced',
    'unanswerable',
    'unanswered',
    'unanticipated',
    'unapologetic',
    'unappareled',
    'unapparent',
    'unappealable',
    'unappealing',
    'unappeasable',
    'unappendaged',
    'unappetising',
    'unappetisingness',
    'unappetizing',
    'unappetizingness',
    'unappreciated',
    'unappreciative',
    'unapprehensive',
    'unapproachability',
    'unapproachable',
    'unarbitrary',
    'unarguable',
    'unargumentative',
    'unarm',
    'unarmed',
    'unarmored',
    'unarmoured',
    'unarticulate',
    'unarticulated',
    'unartistic',
    'unary',
    'unascertainable',
    'unascribable',
    'unashamed',
    'unasked',
    'unassailable',
    'unassertive',
    'unassertiveness',
    'unassignable',
    'unassigned',
    'unassisted',
    'unassuming',
    'unassumingness',
    'unassured',
    'unasterisked',
    'unattached',
    'unattackable',
    'unattainable',
    'unattainableness',
    'unattended',
    'unattired',
    'unattractive',
    'unattractiveness',
    'unattributable',
    'unau',
    'unauthentic',
    'unauthorised',
    'unauthorized',
    'unavailability',
    'unavailable',
    'unavailing',
    'unavenged',
    'unavoidability',
    'unavoidable',
    'unavoidably',
    'unavowed',
    'unawakened',
    'unaware',
    'unawareness',
    'unawares',
    'unawed',
    'unbacked',
    'unbaffled',
    'unbalance',
    'unbalanced',
    'unbalances',
    'unbalconied',
    'unbanded',
    'unbaptised',
    'unbaptized',
    'unbar',
    'unbarred',
    'unbarreled',
    'unbarrelled',
    'unbearable',
    'unbearably',
    'unbeatable',
    'unbeaten',
    'unbecoming',
    'unbecomingness',
    'unbefitting',
    'unbeknown',
    'unbeknownst',
    'unbelief',
    'unbelievable',
    'unbelievably',
    'unbelievers',
    'unbelieving',
    'unbeloved',
    'unbelt',
    'unbelted',
    'unbend',
    'unbendable',
    'unbending',
    'unbeneficed',
    'unbent',
    'unbiased',
    'unbiassed',
    'unbigoted',
    'unbind',
    'unbitter',
    'unbleached',
    'unblemished',
    'unblended',
    'unblessed',
    'unblinking',
    'unblock',
    'unblocked',
    'unbloody',
    'unblushing',
    'unbodied',
    'unbolt',
    'unbolted',
    'unbooked',
    'unbordered',
    'unborn',
    'unbosom',
    'unbound',
    'unbounded',
    'unboundedness',
    'unbowed',
    'unbox',
    'unbrace',
    'unbraced',
    'unbraid',
    'unbrainwashed',
    'unbranched',
    'unbranching',
    'unbranded',
    'unbreakable',
    'unbreakableness',
    'unbridgeable',
    'unbridle',
    'unbridled',
    'unbroken',
    'unbrushed',
    'unbuckle',
    'unburden',
    'unburdened',
    'unburied',
    'unburned',
    'unburnished',
    'unbutton',
    'unbuttoned',
    'uncamphorated',
    'uncannily',
    'uncanny',
    'uncapped',
    'uncarbonated',
    'uncaring',
    'uncarpeted',
    'uncarved',
    'uncase',
    'uncastrated',
    'uncategorised',
    'uncategorized',
    'uncaulked',
    'uncaused',
    'unceasing',
    'uncensored',
    'unceremonial',
    'unceremonious',
    'unceremoniously',
    'unceremoniousness',
    'uncertain',
    'uncertainly',
    'uncertainness',
    'uncertainty',
    'uncertified',
    'unchain',
    'unchained',
    'unchallengeable',
    'unchallenged',
    'unchangeability',
    'unchangeable',
    'unchangeableness',
    'unchanged',
    'unchanging',
    'unchangingness',
    'uncharacteristic',
    'uncharged',
    'uncharitable',
    'uncharted',
    'unchartered',
    'unchaste',
    'uncheckable',
    'unchecked',
    'uncheerful',
    'uncheerfulness',
    'unchewable',
    'unchivalrous',
    'unchristian',
    'unchristianly',
    'unchristlike',
    'unchurch',
    'uncial',
    'uncivil',
    'uncivilised',
    'uncivilized',
    'unclad',
    'unclaimed',
    'unclasp',
    'unclasped',
    'unclassifiable',
    'unclassified',
    'uncle',
    'unclean',
    'uncleanliness',
    'uncleanly',
    'uncleanness',
    'unclear',
    'uncleared',
    'unclearness',
    'unclimbable',
    'unclip',
    'unclipped',
    'uncloak',
    'unclog',
    'unclogged',
    'unclothe',
    'unclothed',
    'unclouded',
    'uncloudedness',
    'unclutter',
    'uncluttered',
    'uncoated',
    'uncoerced',
    'uncoil',
    'uncoiled',
    'uncoiling',
    'uncoils',
    'uncollected',
    'uncollectible',
    'uncolored',
    'uncoloured',
    'uncolumned',
    'uncombable',
    'uncombed',
    'uncombined',
    'uncomely',
    'uncomfortable',
    'uncomfortableness',
    'uncomfortably',
    'uncommercial',
    'uncommercialised',
    'uncommercialized',
    'uncommitted',
    'uncommon',
    'uncommonly',
    'uncommonness',
    'uncommunicative',
    'uncommunicativeness',
    'uncomparable',
    'uncompartmented',
    'uncompassionate',
    'uncompensated',
    'uncompetitive',
    'uncomplaining',
    'uncomplete',
    'uncompleted',
    'uncomplicated',
    'uncomplimentary',
    'uncompounded',
    'uncomprehended',
    'uncomprehending',
    'uncomprehensible',
    'uncompress',
    'uncompressed',
    'uncompromising',
    'uncompromisingly',
    'unconcealed',
    'unconcern',
    'unconcerned',
    'unconditional',
    'unconditioned',
    'unconfessed',
    'unconfined',
    'unconfirmed',
    'unconformable',
    'unconformist',
    'unconfused',
    'uncongenial',
    'uncongeniality',
    'unconnected',
    'unconnectedness',
    'unconquerable',
    'unconquered',
    'unconscientious',
    'unconscientiousness',
    'unconscionable',
    'unconscious',
    'unconsciously',
    'unconsciousness',
    'unconsecrated',
    'unconsidered',
    'unconsolable',
    'unconsolidated',
    'unconstipated',
    'unconstitutional',
    'unconstrained',
    'unconstraint',
    'unconstricted',
    'unconstructive',
    'unconsumed',
    'unconsummated',
    'uncontaminated',
    'uncontaminating',
    'uncontested',
    'uncontrived',
    'uncontrollable',
    'uncontrollably',
    'uncontrolled',
    'uncontroversial',
    'unconventional',
    'unconventionality',
    'unconverted',
    'unconvertible',
    'unconvinced',
    'unconvincing',
    'uncooked',
    'uncool',
    'uncooperative',
    'uncoordinated',
    'uncordial',
    'uncork',
    'uncorrectable',
    'uncorrected',
    'uncorrelated',
    'uncorroborated',
    'uncorrupted',
    'uncounted',
    'uncouple',
    'uncoupled',
    'uncousinly',
    'uncouth',
    'uncouthness',
    'uncover',
    'uncovered',
    'uncovering',
    'uncrannied',
    'uncrate',
    'uncreased',
    'uncreative',
    'uncreativeness',
    'uncritical',
    'uncritically',
    'uncropped',
    'uncross',
    'uncrossed',
    'uncrowded',
    'uncrowned',
    'uncrystallised',
    'uncrystallized',
    'unction',
    'unctuous',
    'unctuousness',
    'uncultivable',
    'uncultivatable',
    'uncultivated',
    'uncultured',
    'uncurbed',
    'uncured',
    'uncurl',
    'uncurled',
    'uncurling',
    'uncurtained',
    'uncurved',
    'uncurving',
    'uncus',
    'uncut',
    'undamaged',
    'undatable',
    'undated',
    'undaunted',
    'undecagon',
    'undeceive',
    'undeceived',
    'undecided',
    'undecipherable',
    'undeciphered',
    'undeclared',
    'undecomposable',
    'undecomposed',
    'undecorated',
    'undedicated',
    'undefeated',
    'undefendable',
    'undefended',
    'undeferential',
    'undefiled',
    'undefinable',
    'undefined',
    'undelineated',
    'undemanding',
    'undemocratic',
    'undemonstrative',
    'undeniable',
    'undenominational',
    'undependability',
    'undependable',
    'undependableness',
    'undepicted',
    'under',
    'underachieve',
    'underachievement',
    'underachiever',
    'underact',
    'underactive',
    'underage',
    'underarm',
    'underbellies',
    'underbelly',
    'underbid',
    'underbodice',
    'underbody',
    'underboss',
    'underbred',
    'underbrush',
    'undercarriage',
    'undercharge',
    'underclass',
    'underclassman',
    'underclothed',
    'underclothes',
    'underclothing',
    'undercoat',
    'undercoated',
    'undercover',
    'undercurrent',
    'undercurrents',
    'undercut',
    'underdevelop',
    'underdeveloped',
    'underdevelopment',
    'underdog',
    'underdone',
    'underdrawers',
    'underdress',
    'underdressed',
    'undereducated',
    'underemployed',
    'underestimate',
    'underestimated',
    'underestimates',
    'underestimation',
    'underevaluation',
    'underexpose',
    'underexposed',
    'underexposure',
    'underfed',
    'underfelt',
    'underfoot',
    'underframe',
    'underfur',
    'undergarment',
    'undergarments',
    'undergird',
    'undergo',
    'undergoes',
    'undergoing',
    'undergone',
    'undergrad',
    'undergraduate',
    'undergraduates',
    'undergrew',
    'underground',
    'undergrow',
    'undergrowth',
    'underhand',
    'underhanded',
    'underhandedly',
    'underhung',
    'underivative',
    'underived',
    'underlaid',
    'underlay',
    'underlayment',
    'underlie',
    'underlies',
    'underline',
    'underling',
    'underlining',
    'underlip',
    'underlying',
    'undermanned',
    'undermentioned',
    'undermine',
    'undermined',
    'undermines',
    'undermining',
    'underneath',
    'undernourish',
    'undernourished',
    'undernourishment',
    'underpants',
    'underpart',
    'underparts',
    'underpass',
    'underpay',
    'underpayment',
    'underperform',
    'underperformed',
    'underperformer',
    'underpin',
    'underplay',
    'underpopulated',
    'underprice',
    'underprivileged',
    'underproduce',
    'underproducing',
    'underproduction',
    'underquote',
    'underrate',
    'underrating',
    'underreckoning',
    'underrun',
    'underscore',
    'underscoring',
    'undersea',
    'underseal',
    'undersealed',
    'undersecretary',
    'undersell',
    'underseller',
    'undersexed',
    'undershirt',
    'undershoot',
    'undershot',
    'undershrub',
    'undershrubs',
    'underside',
    'undersides',
    'undersign',
    'undersize',
    'undersized',
    'underskirt',
    'underslung',
    'undersoil',
    'underspend',
    'understaffed',
    'understand',
    'understandability',
    'understandable',
    'understanding',
    'understands',
    'understate',
    'understated',
    'understatement',
    'understating',
    'understock',
    'understood',
    'understructure',
    'understudy',
    'undersurface',
    'undersurfaces',
    'undertake',
    'undertaken',
    'undertaker',
    'undertakes',
    'undertaking',
    'undertide',
    'undertone',
    'undertow',
    'undervaluation',
    'undervalue',
    'underwater',
    'underway',
    'underwear',
    'underweight',
    'underwent',
    'underwing',
    'underwings',
    'underwood',
    'underworld',
    'underwrite',
    'underwriter',
    'underwriters',
    'underwriting',
    'undescended',
    'undescriptive',
    'undeserved',
    'undeservedly',
    'undeserving',
    'undesigned',
    'undesirability',
    'undesirable',
    'undesirably',
    'undesired',
    'undesiring',
    'undesirous',
    'undestroyable',
    'undetectable',
    'undetected',
    'undeterminable',
    'undetermined',
    'undeterred',
    'undeveloped',
    'undeviating',
    'undiagnosable',
    'undiagnosed',
    'undid',
    'undies',
    'undifferentiated',
    'undigested',
    'undignified',
    'undiluted',
    'undiminished',
    'undimmed',
    'undine',
    'undiplomatic',
    'undirected',
    'undiscerning',
    'undischarged',
    'undiscipline',
    'undisciplined',
    'undisclosed',
    'undiscouraged',
    'undiscoverable',
    'undiscovered',
    'undiscriminating',
    'undisguised',
    'undismayed',
    'undisputable',
    'undisputed',
    'undissolved',
    'undistinguishable',
    'undistinguished',
    'undistorted',
    'undistributed',
    'undisturbed',
    'undiversified',
    'undividable',
    'undivided',
    'undo',
    'undoable',
    'undock',
    'undocked',
    'undocumented',
    'undoer',
    'undoers',
    'undogmatic',
    'undogmatical',
    'undoing',
    'undomestic',
    'undomesticated',
    'undone',
    'undrained',
    'undramatic',
    'undrape',
    'undraped',
    'undrawn',
    'undreamed',
    'undreamt',
    'undress',
    'undressed',
    'undresses',
    'undressing',
    'undried',
    'undrinkable',
    'undset',
    'undue',
    'undulant',
    'undulate',
    'undulating',
    'undulation',
    'undulations',
    'undulatory',
    'unduly',
    'undutiful',
    'undutifulness',
    'undyed',
    'undying',
    'undynamic',
    'uneager',
    'unearned',
    'unearth',
    'unearthed',
    'unearthly',
    'unease',
    'uneasiness',
    'uneasy',
    'uneatable',
    'uneconomic',
    'uneconomical',
    'unedifying',
    'unedited',
    'uneducated',
    'uneffective',
    'unelaborate',
    'unelaborated',
    'unembarrassed',
    'unembellished',
    'unembodied',
    'unemotional',
    'unemotionality',
    'unemphatic',
    'unemployable',
    'unemployed',
    'unemployment',
    'unenclosed',
    'unencouraging',
    'unencumbered',
    'unended',
    'unending',
    'unendowed',
    'unendurable',
    'unenergetic',
    'unenforceable',
    'unenforced',
    'unengaged',
    'unenlightened',
    'unenlightening',
    'unenlightenment',
    'unenlivened',
    'unenterprising',
    'unenthusiastic',
    'unentitled',
    'unenviable',
    'unequal',
    'unequaled',
    'unequalised',
    'unequalized',
    'unequalled',
    'unequipped',
    'unequivocal',
    'unequivocally',
    'unequivocalness',
    'unerasable',
    'unerect',
    'unerring',
    'unerringly',
    'unesco',
    'unessential',
    'unestablished',
    'unethical',
    'unethically',
    'uneven',
    'unevenly',
    'unevenness',
    'uneventful',
    'unexacting',
    'unexampled',
    'unexceeded',
    'unexcelled',
    'unexceptionable',
    'unexceptional',
    'unexchangeability',
    'unexchangeable',
    'unexcitable',
    'unexcited',
    'unexciting',
    'unexclusive',
    'unexcused',
    'unexhausted',
    'unexpansive',
    'unexpected',
    'unexpectedly',
    'unexpectedness',
    'unexpendable',
    'unexpended',
    'unexpired',
    'unexplainable',
    'unexplained',
    'unexploded',
    'unexploited',
    'unexplorative',
    'unexploratory',
    'unexplored',
    'unexportable',
    'unexpressed',
    'unexpressible',
    'unexpressive',
    'unexpurgated',
    'unextended',
    'unfaceted',
    'unfading',
    'unfailing',
    'unfair',
    'unfairly',
    'unfairness',
    'unfaithful',
    'unfaithfulness',
    'unfaltering',
    'unfamiliar',
    'unfamiliarity',
    'unfashionable',
    'unfasten',
    'unfastened',
    'unfastener',
    'unfastening',
    'unfastens',
    'unfastidious',
    'unfathomable',
    'unfathomed',
    'unfattened',
    'unfavorable',
    'unfavorableness',
    'unfavorably',
    'unfavourable',
    'unfavourableness',
    'unfearing',
    'unfeasibility',
    'unfeasible',
    'unfeathered',
    'unfed',
    'unfeeling',
    'unfeelingness',
    'unfeigned',
    'unfeminine',
    'unfenced',
    'unfermented',
    'unfertile',
    'unfertilised',
    'unfertilized',
    'unfettered',
    'unfilled',
    'unfilmed',
    'unfinished',
    'unfirm',
    'unfit',
    'unfitness',
    'unfitting',
    'unfixed',
    'unflagging',
    'unflappable',
    'unflattering',
    'unflavored',
    'unflavoured',
    'unflawed',
    'unfledged',
    'unflinching',
    'unfluctuating',
    'unflurried',
    'unflustered',
    'unfocused',
    'unfocussed',
    'unfold',
    'unfolding',
    'unfolds',
    'unforbearing',
    'unforced',
    'unforceful',
    'unforeseeable',
    'unforeseen',
    'unforesightful',
    'unforested',
    'unforethoughtful',
    'unforfeitable',
    'unforgettable',
    'unforgivable',
    'unforgiving',
    'unformed',
    'unfortunate',
    'unfortunately',
    'unfounded',
    'unframed',
    'unfree',
    'unfreeze',
    'unfrequented',
    'unfretted',
    'unfriendliness',
    'unfriendly',
    'unfrightened',
    'unfrock',
    'unfrosted',
    'unfrozen',
    'unfruitful',
    'unfueled',
    'unfulfilled',
    'unfunctional',
    'unfunded',
    'unfunny',
    'unfurl',
    'unfurnished',
    'unfurrowed',
    'ungainliness',
    'ungainly',
    'ungallant',
    'ungarbed',
    'ungarmented',
    'ungathered',
    'ungeared',
    'ungenerous',
    'ungentle',
    'ungentlemanlike',
    'ungentlemanly',
    'ungetatable',
    'unglamorous',
    'unglamourous',
    'unglazed',
    'ungodliness',
    'ungodly',
    'ungovernable',
    'ungoverned',
    'ungraceful',
    'ungracefulness',
    'ungracious',
    'ungraciousness',
    'ungraded',
    'ungrammatical',
    'ungrasped',
    'ungrateful',
    'ungratefulness',
    'ungratified',
    'ungratifying',
    'ungreased',
    'ungregarious',
    'ungroomed',
    'ungrudging',
    'ungual',
    'unguaranteed',
    'unguarded',
    'unguent',
    'unguiculata',
    'unguiculate',
    'unguiculated',
    'unguided',
    'unguis',
    'ungulata',
    'ungulate',
    'ungulated',
    'ungulates',
    'ungummed',
    'ungusseted',
    'unhallow',
    'unhallowed',
    'unhampered',
    'unhand',
    'unhappiness',
    'unhappy',
    'unhardened',
    'unharmed',
    'unharmonious',
    'unharness',
    'unhatched',
    'unhazardous',
    'unheaded',
    'unhealed',
    'unhealthful',
    'unhealthfulness',
    'unhealthiness',
    'unhealthy',
    'unhearable',
    'unheard',
    'unhearing',
    'unheated',
    'unheeded',
    'unheeding',
    'unhelpful',
    'unhelpfulness',
    'unheralded',
    'unhesitating',
    'unhewn',
    'unhindered',
    'unhinge',
    'unhinged',
    'unhitch',
    'unholiness',
    'unholy',
    'unhomogenised',
    'unhomogenized',
    'unhook',
    'unhoped',
    'unhopeful',
    'unhorse',
    'unhuman',
    'unhumorous',
    'unhurried',
    'unhurriedness',
    'unhurt',
    'unhygienic',
    'uniat',
    'uniate',
    'unicameral',
    'unicef',
    'unicellular',
    'unicorn',
    'unicuspid',
    'unicycle',
    'unicyclist',
    'unidentifiable',
    'unidentified',
    'unidimensional',
    'unidirectional',
    'unifacial',
    'unification',
    'unified',
    'unifies',
    'unifilar',
    'unifoliate',
    'uniform',
    'uniformed',
    'uniformise',
    'uniformity',
    'uniformize',
    'uniformly',
    'uniformness',
    'uniforms',
    'unify',
    'unifying',
    'unilateral',
    'unilateralism',
    'unilateralist',
    'unilluminated',
    'unilluminating',
    'unimaginable',
    'unimaginably',
    'unimaginative',
    'unimaginatively',
    'unimagined',
    'unimodal',
    'unimpaired',
    'unimpassioned',
    'unimpeachable',
    'unimpeded',
    'unimportance',
    'unimportant',
    'unimposing',
    'unimpregnated',
    'unimpressed',
    'unimpressionable',
    'unimpressive',
    'unimprisoned',
    'unimproved',
    'unincorporated',
    'unindustrialised',
    'unindustrialized',
    'uninebriated',
    'uninfected',
    'uninflected',
    'uninfluenced',
    'uninfluential',
    'uninformative',
    'uninformed',
    'uninhabitable',
    'uninhabited',
    'uninhibited',
    'uninitiate',
    'uninitiated',
    'uninjectable',
    'uninjured',
    'uninominal',
    'uninquiring',
    'uninquisitive',
    'uninspired',
    'uninspiring',
    'uninstructed',
    'uninstructive',
    'uninsurability',
    'uninsurable',
    'uninsured',
    'unintegrated',
    'unintelligent',
    'unintelligibility',
    'unintelligible',
    'unintelligibly',
    'unintended',
    'unintentional',
    'unintentionally',
    'uninterested',
    'uninteresting',
    'uninterestingness',
    'uninterrupted',
    'unintimidated',
    'unintoxicated',
    'unintrusive',
    'uninucleate',
    'uninventive',
    'uninvited',
    'uninviting',
    'uninvolved',
    'unio',
    'union',
    'unionidae',
    'unionisation',
    'unionise',
    'unionised',
    'unionism',
    'unionist',
    'unionization',
    'unionize',
    'unionized',
    'unions',
    'uniovular',
    'uniovulate',
    'uniparous',
    'unipolar',
    'unique',
    'uniquely',
    'uniqueness',
    'unironed',
    'unisex',
    'unisexual',
    'unison',
    'unit',
    'unitard',
    'unitarian',
    'unitarianism',
    'unitary',
    'unite',
    'united',
    'unites',
    'uniting',
    'unitisation',
    'unitise',
    'unitization',
    'unitize',
    'unitized',
    'units',
    'unity',
    'univalent',
    'univalve',
    'universal',
    'universalise',
    'universalism',
    'universalist',
    'universalistic',
    'universality',
    'universalize',
    'universally',
    'universals',
    'universe',
    'universities',
    'university',
    'univocal',
    'unix',
    'unjointed',
    'unjust',
    'unjustifiable',
    'unjustified',
    'unjustly',
    'unjustness',
    'unkempt',
    'unkemptness',
    'unkept',
    'unkeyed',
    'unkind',
    'unkindest',
    'unkindled',
    'unkindly',
    'unkindness',
    'unknot',
    'unknowable',
    'unknowing',
    'unknowingly',
    'unknowingness',
    'unknowledgeable',
    'unknown',
    'unlabeled',
    'unlabelled',
    'unlace',
    'unlaced',
    'unlade',
    'unladylike',
    'unlamented',
    'unlash',
    'unlatched',
    'unlaureled',
    'unlaurelled',
    'unlawful',
    'unlawfully',
    'unlawfulness',
    'unlax',
    'unleaded',
    'unlearn',
    'unlearned',
    'unleash',
    'unleavened',
    'unless',
    'unlettered',
    'unlicenced',
    'unlicensed',
    'unlifelike',
    'unlighted',
    'unlikable',
    'unlike',
    'unlikeable',
    'unlikelihood',
    'unlikeliness',
    'unlikely',
    'unlikeness',
    'unlimited',
    'unlined',
    'unlipped',
    'unlisted',
    'unlit',
    'unliterary',
    'unlittered',
    'unlivable',
    'unlive',
    'unliveable',
    'unliveried',
    'unload',
    'unloaded',
    'unloading',
    'unloads',
    'unlobed',
    'unlocated',
    'unlock',
    'unlocked',
    'unlogical',
    'unlooked',
    'unloose',
    'unloosen',
    'unlovable',
    'unloved',
    'unlovely',
    'unloving',
    'unlubricated',
    'unlucky',
    'unmade',
    'unmake',
    'unmalicious',
    'unmalleability',
    'unmalleable',
    'unmalted',
    'unman',
    'unmanageable',
    'unmanageableness',
    'unmanful',
    'unmanlike',
    'unmanliness',
    'unmanly',
    'unmanned',
    'unmannered',
    'unmannerly',
    'unmanning',
    'unmapped',
    'unmarked',
    'unmarketable',
    'unmarred',
    'unmarried',
    'unmask',
    'unmasked',
    'unmasking',
    'unmatchable',
    'unmatched',
    'unmated',
    'unmeaning',
    'unmeasurable',
    'unmeasured',
    'unmechanical',
    'unmechanised',
    'unmechanized',
    'unmediated',
    'unmedical',
    'unmedicative',
    'unmedicinal',
    'unmelodic',
    'unmelodious',
    'unmelted',
    'unmemorable',
    'unmentionable',
    'unmercenary',
    'unmerchantable',
    'unmerciful',
    'unmercifulness',
    'unmerited',
    'unmeritorious',
    'unmethodical',
    'unmilitary',
    'unmindful',
    'unmindfulness',
    'unmined',
    'unmingled',
    'unmistakable',
    'unmitigable',
    'unmitigated',
    'unmixable',
    'unmixed',
    'unmoderated',
    'unmodernised',
    'unmodernized',
    'unmodifiable',
    'unmodified',
    'unmodulated',
    'unmolested',
    'unmortgaged',
    'unmotivated',
    'unmotorised',
    'unmotorized',
    'unmourned',
    'unmovable',
    'unmoved',
    'unmoving',
    'unmown',
    'unmusical',
    'unmutilated',
    'unmuzzle',
    'unmyelinated',
    'unnameable',
    'unnamed',
    'unnatural',
    'unnaturalised',
    'unnaturalized',
    'unnaturally',
    'unnaturalness',
    'unnavigable',
    'unnecessarily',
    'unnecessary',
    'unneeded',
    'unneighborliness',
    'unneighborly',
    'unneighbourly',
    'unnerve',
    'unnerved',
    'unnerving',
    'unneurotic',
    'unnilquadium',
    'unnotched',
    'unnoted',
    'unnoticeable',
    'unnoticeableness',
    'unnoticed',
    'unnourished',
    'unnumberable',
    'unnumbered',
    'unnumerable',
    'unobjectionable',
    'unobjective',
    'unobligated',
    'unobliging',
    'unobservable',
    'unobservant',
    'unobserved',
    'unobstructed',
    'unobtainable',
    'unobtrusive',
    'unobtrusively',
    'unobtrusiveness',
    'unobvious',
    'unoccupied',
    'unoffending',
    'unofficial',
    'unofficially',
    'unoiled',
    'unopen',
    'unopened',
    'unopposable',
    'unopposed',
    'unordered',
    'unorganised',
    'unorganized',
    'unoriented',
    'unoriginal',
    'unoriginality',
    'unoriginally',
    'unornamented',
    'unorthodox',
    'unorthodoxy',
    'unostentatious',
    'unowned',
    'unoxygenated',
    'unpack',
    'unpackaged',
    'unpaid',
    'unpainful',
    'unpaintable',
    'unpainted',
    'unpaired',
    'unpalatability',
    'unpalatable',
    'unpalatableness',
    'unparallel',
    'unparalleled',
    'unpardonable',
    'unparented',
    'unparliamentary',
    'unpartitioned',
    'unpassable',
    'unpasteurised',
    'unpasteurized',
    'unpatented',
    'unpatriotic',
    'unpatronised',
    'unpatronized',
    'unpatterned',
    'unpaved',
    'unpeaceable',
    'unpeaceful',
    'unpeopled',
    'unperceivable',
    'unperceived',
    'unperceiving',
    'unperceptive',
    'unperceptiveness',
    'unperformed',
    'unpermed',
    'unpermissive',
    'unpermissiveness',
    'unperplexed',
    'unperson',
    'unpersuadable',
    'unpersuaded',
    'unpersuasive',
    'unpersuasiveness',
    'unperturbed',
    'unpick',
    'unpictured',
    'unpicturesque',
    'unpierced',
    'unpigmented',
    'unpillared',
    'unpin',
    'unpitying',
    'unplaced',
    'unplanned',
    'unplanted',
    'unplayable',
    'unplayful',
    'unpleasant',
    'unpleasantly',
    'unpleasantness',
    'unpleasing',
    'unpleasingness',
    'unpledged',
    'unploughed',
    'unplowed',
    'unplug',
    'unplumbed',
    'unpointed',
    'unpointedness',
    'unpoliced',
    'unpolished',
    'unpolitical',
    'unpolluted',
    'unpompous',
    'unpopular',
    'unpopularity',
    'unpopulated',
    'unportable',
    'unposed',
    'unpotted',
    'unpowered',
    'unpracticed',
    'unpractised',
    'unprecedented',
    'unpredictability',
    'unpredictable',
    'unpredictably',
    'unpredicted',
    'unpredictive',
    'unprejudiced',
    'unpremeditated',
    'unprepared',
    'unprepossessing',
    'unpresentable',
    'unpresidential',
    'unpressed',
    'unpretending',
    'unpretentious',
    'unpretentiousness',
    'unpreventable',
    'unpriestly',
    'unprincipled',
    'unprintable',
    'unproblematic',
    'unprocessed',
    'unprocurable',
    'unproductive',
    'unproductiveness',
    'unprofessional',
    'unprofitability',
    'unprofitable',
    'unprofitableness',
    'unprogressive',
    'unpromised',
    'unpromising',
    'unprompted',
    'unpronounceable',
    'unprophetic',
    'unpropitious',
    'unpropitiousness',
    'unprotected',
    'unprotectedness',
    'unprotective',
    'unprovable',
    'unproved',
    'unproven',
    'unprovided',
    'unprovocative',
    'unprovoked',
    'unprovoking',
    'unpublishable',
    'unpublished',
    'unpunctual',
    'unpunished',
    'unpurified',
    'unq',
    'unqualified',
    'unquenchable',
    'unquestionability',
    'unquestionable',
    'unquestionableness',
    'unquestioned',
    'unquestioning',
    'unquestioningly',
    'unquiet',
    'unquotable',
    'unraised',
    'unranked',
    'unratable',
    'unratified',
    'unravel',
    'unraveled',
    'unraveler',
    'unravelled',
    'unraveller',
    'unravelling',
    'unreachable',
    'unreached',
    'unreactive',
    'unread',
    'unreadable',
    'unready',
    'unreal',
    'unrealised',
    'unrealism',
    'unrealistic',
    'unrealistically',
    'unreality',
    'unrealizable',
    'unrealized',
    'unreason',
    'unreasonable',
    'unreasonably',
    'unreasoning',
    'unreassuring',
    'unreceptive',
    'unreciprocated',
    'unrecognisable',
    'unrecognised',
    'unrecognizable',
    'unrecognized',
    'unreconcilable',
    'unreconciled',
    'unreconstructed',
    'unrecorded',
    'unrecoverable',
    'unredeemable',
    'unredeemed',
    'unreduced',
    'unreel',
    'unrefined',
    'unreflected',
    'unreflective',
    'unreformable',
    'unreformed',
    'unrefreshed',
    'unregenerate',
    'unregenerated',
    'unregistered',
    'unregretful',
    'unregretting',
    'unregularity',
    'unregulated',
    'unrehearsed',
    'unrelated',
    'unrelatedness',
    'unrelaxed',
    'unreleased',
    'unrelenting',
    'unreliability',
    'unreliable',
    'unreliableness',
    'unrelieved',
    'unremarkable',
    'unremarked',
    'unremedied',
    'unremitting',
    'unremorseful',
    'unremunerative',
    'unrenewable',
    'unrenewed',
    'unrentable',
    'unrepaired',
    'unrepeatable',
    'unrepentant',
    'unrepenting',
    'unreplaceable',
    'unreportable',
    'unreported',
    'unrepresentative',
    'unrepressed',
    'unreproducible',
    'unrequested',
    'unrequited',
    'unresentful',
    'unreserved',
    'unresistant',
    'unresisting',
    'unresolvable',
    'unresolved',
    'unrespectability',
    'unrespectable',
    'unresponsive',
    'unresponsiveness',
    'unrest',
    'unrested',
    'unrestrained',
    'unrestrainedly',
    'unrestraint',
    'unrestricted',
    'unrestrictive',
    'unretentive',
    'unretrievable',
    'unrevealed',
    'unreverberant',
    'unrevised',
    'unrevived',
    'unrewarded',
    'unrewarding',
    'unrhetorical',
    'unrhymed',
    'unrhythmic',
    'unrhythmical',
    'unrifled',
    'unrigged',
    'unrighteous',
    'unrighteousness',
    'unrimed',
    'unripe',
    'unripened',
    'unrivaled',
    'unrivalled',
    'unroll',
    'unromantic',
    'unroofed',
    'unrouged',
    'unruffled',
    'unruliness',
    'unruly',
    'unsaddle',
    'unsaddled',
    'unsafe',
    'unsaid',
    'unsalable',
    'unsalaried',
    'unsaleable',
    'unsalted',
    'unsanctification',
    'unsanctified',
    'unsanctify',
    'unsanctioned',
    'unsanded',
    'unsanitariness',
    'unsanitary',
    'unsaponified',
    'unsarcastic',
    'unsated',
    'unsatiable',
    'unsatiated',
    'unsatisfactoriness',
    'unsatisfactory',
    'unsatisfiable',
    'unsatisfied',
    'unsatisfying',
    'unsaturated',
    'unsaved',
    'unsavoriness',
    'unsavory',
    'unsavoury',
    'unsay',
    'unscalable',
    'unscathed',
    'unscheduled',
    'unscholarly',
    'unschooled',
    'unscientific',
    'unscramble',
    'unscrew',
    'unscrewing',
    'unscript',
    'unscripted',
    'unscrupulous',
    'unscrupulously',
    'unscrupulousness',
    'unseal',
    'unsealed',
    'unseamanlike',
    'unseamed',
    'unseasonable',
    'unseasonableness',
    'unseasoned',
    'unseat',
    'unseaworthy',
    'unsectarian',
    'unsecured',
    'unseductive',
    'unseeable',
    'unseeded',
    'unseeing',
    'unseemliness',
    'unseemly',
    'unseen',
    'unsegmented',
    'unsegregated',
    'unselected',
    'unselective',
    'unselfconscious',
    'unselfconsciousness',
    'unselfish',
    'unselfishly',
    'unselfishness',
    'unsensational',
    'unsent',
    'unsentimental',
    'unserviceable',
    'unservile',
    'unsettle',
    'unsettled',
    'unsex',
    'unsexed',
    'unsexy',
    'unshackled',
    'unshaded',
    'unshadowed',
    'unshakable',
    'unshakeable',
    'unshaken',
    'unshaped',
    'unshapely',
    'unshapen',
    'unshared',
    'unsharpened',
    'unshaved',
    'unshaven',
    'unsheared',
    'unsheathe',
    'unsheathed',
    'unshelled',
    'unshielded',
    'unshockable',
    'unshod',
    'unshoed',
    'unshorn',
    'unshrinkable',
    'unshrinking',
    'unshuttered',
    'unsighted',
    'unsightliness',
    'unsightly',
    'unsigned',
    'unsilenced',
    'unsimilarity',
    'unsinkable',
    'unsized',
    'unskilled',
    'unskillful',
    'unskillfulness',
    'unsleeping',
    'unsloped',
    'unsmiling',
    'unsmooth',
    'unsmoothed',
    'unsnarl',
    'unsnarled',
    'unsnarling',
    'unsociability',
    'unsociable',
    'unsociableness',
    'unsocial',
    'unsoiled',
    'unsold',
    'unsolder',
    'unsoldierly',
    'unsolicited',
    'unsoluble',
    'unsolvability',
    'unsolvable',
    'unsolved',
    'unsophisticated',
    'unsorted',
    'unsought',
    'unsound',
    'unsoundable',
    'unsounded',
    'unsoundness',
    'unsoured',
    'unsown',
    'unspaced',
    'unsparing',
    'unspeakable',
    'unspecialised',
    'unspecialized',
    'unspecific',
    'unspecified',
    'unspectacular',
    'unspell',
    'unspent',
    'unspoiled',
    'unspoilt',
    'unspoken',
    'unsporting',
    'unsportsmanlike',
    'unspotted',
    'unstable',
    'unstableness',
    'unstaged',
    'unstained',
    'unstaple',
    'unstarred',
    'unstated',
    'unstatesmanlike',
    'unsteadily',
    'unsteadiness',
    'unsteady',
    'unsterilised',
    'unsterilized',
    'unstilted',
    'unstimulating',
    'unstinted',
    'unstinting',
    'unstirred',
    'unstoppable',
    'unstoppered',
    'unstrain',
    'unstrained',
    'unstrap',
    'unstratified',
    'unstressed',
    'unstring',
    'unstructured',
    'unstrung',
    'unstuck',
    'unstudied',
    'unstudious',
    'unstuff',
    'unstuffed',
    'unstylish',
    'unsuasible',
    'unsubdivided',
    'unsubduable',
    'unsubdued',
    'unsubmissive',
    'unsubstantial',
    'unsubstantialise',
    'unsubstantialize',
    'unsubstantiated',
    'unsubtle',
    'unsuccessful',
    'unsuccessfully',
    'unsufferable',
    'unsugared',
    'unsuitability',
    'unsuitable',
    'unsuitableness',
    'unsuitably',
    'unsuited',
    'unsullied',
    'unsung',
    'unsupervised',
    'unsupportable',
    'unsupported',
    'unsupportive',
    'unsuppressed',
    'unsuprised',
    'unsure',
    'unsurmountable',
    'unsurpassable',
    'unsurpassed',
    'unsurprised',
    'unsurprising',
    'unsusceptibility',
    'unsusceptible',
    'unsuspected',
    'unsuspecting',
    'unsuspicious',
    'unswayed',
    'unsweet',
    'unsweetened',
    'unswept',
    'unswerving',
    'unsworn',
    'unsyllabic',
    'unsyllabled',
    'unsymbolic',
    'unsymmetric',
    'unsymmetrical',
    'unsympathetic',
    'unsympathising',
    'unsympathizing',
    'unsynchronised',
    'unsynchronized',
    'unsynchronous',
    'unsystematic',
    'untactful',
    'untagged',
    'untainted',
    'untalented',
    'untalkative',
    'untamed',
    'untangle',
    'untangled',
    'untangling',
    'untanned',
    'untaped',
    'untapped',
    'untarnished',
    'untasted',
    'untaught',
    'untaxed',
    'unteach',
    'untechnical',
    'untellable',
    'untempered',
    'untempting',
    'untenable',
    'untenanted',
    'untended',
    'untermeyer',
    'untested',
    'untethered',
    'unthankful',
    'unthaw',
    'unthawed',
    'untheatrical',
    'unthematic',
    'unthinkable',
    'unthinking',
    'unthought',
    'unthoughtful',
    'unthoughtfulness',
    'unthreatening',
    'untidiness',
    'untidy',
    'untie',
    'untied',
    'untier',
    'until',
    'untilled',
    'untimbered',
    'untimeliness',
    'untimely',
    'untipped',
    'untired',
    'untiring',
    'untitled',
    'unto',
    'untoasted',
    'untold',
    'untoothed',
    'untouchable',
    'untouched',
    'untoughened',
    'untoward',
    'untraceable',
    'untracked',
    'untraditional',
    'untrained',
    'untrammeled',
    'untrammelled',
    'untransferable',
    'untranslatable',
    'untransmutable',
    'untraveled',
    'untravelled',
    'untraversable',
    'untraversed',
    'untreated',
    'untried',
    'untrimmed',
    'untrod',
    'untrodden',
    'untroubled',
    'untrue',
    'untrustiness',
    'untrusting',
    'untrustworthiness',
    'untrustworthy',
    'untrusty',
    'untruth',
    'untruthful',
    'untruthfulness',
    'untucked',
    'untufted',
    'untune',
    'untuneful',
    'unturned',
    'untutored',
    'untwine',
    'untwist',
    'untwisted',
    'untying',
    'untypical',
    'untypicality',
    'ununbium',
    'ununderstood',
    'ununhexium',
    'ununpentium',
    'ununquadium',
    'ununtrium',
    'unusable',
    'unuseable',
    'unused',
    'unusefulness',
    'unusual',
    'unusually',
    'unusualness',
    'unutterable',
    'unuttered',
    'unvaccinated',
    'unvalued',
    'unvaned',
    'unvanquishable',
    'unvanquished',
    'unvaried',
    'unvariedness',
    'unvarnished',
    'unvarying',
    'unveil',
    'unveiled',
    'unveiling',
    'unvendible',
    'unvented',
    'unventilated',
    'unverbalised',
    'unverbalized',
    'unverifiable',
    'unverified',
    'unversed',
    'unvigilant',
    'unvindictive',
    'unvitrified',
    'unvoiced',
    'unvoluntary',
    'unvulcanised',
    'unvulcanized',
    'unwanted',
    'unwariness',
    'unwarmed',
    'unwarrantable',
    'unwarranted',
    'unwary',
    'unwashed',
    'unwatchful',
    'unwatered',
    'unwavering',
    'unwaxed',
    'unweaned',
    'unwearable',
    'unwearied',
    'unweary',
    'unwearying',
    'unweathered',
    'unweave',
    'unwebbed',
    'unwed',
    'unwedded',
    'unwelcome',
    'unwell',
    'unwellness',
    'unwholesome',
    'unwholesomeness',
    'unwieldiness',
    'unwieldy',
    'unwilled',
    'unwilling',
    'unwillingly',
    'unwillingness',
    'unwind',
    'unwinding',
    'unwire',
    'unwise',
    'unwisely',
    'unwiseness',
    'unwished',
    'unwitting',
    'unwittingly',
    'unwomanly',
    'unwonted',
    'unwooded',
    'unworkable',
    'unworkmanlike',
    'unworldly',
    'unworried',
    'unworthiness',
    'unworthy',
    'unwounded',
    'unwoven',
    'unwrap',
    'unwrapped',
    'unwraps',
    'unwrinkled',
    'unwritten',
    'unyielding',
    'unyieldingness',
    'unyoke',
    'unzip',
    'up',
    'upanishad',
    'upanishads',
    'upbeat',
    'upbound',
    'upbraid',
    'upbraider',
    'upbraiding',
    'upbringing',
    'upcast',
    'upchuck',
    'upcoming',
    'upcountry',
    'upcurved',
    'update',
    'updated',
    'updates',
    'updating',
    'updike',
    'updraft',
    'updrafts',
    'upend',
    'upended',
    'upending',
    'upfield',
    'upfront',
    'upgrade',
    'upgraded',
    'upheaval',
    'upheave',
    'upheld',
    'uphill',
    'uphold',
    'upholder',
    'upholders',
    'upholds',
    'upholster',
    'upholstered',
    'upholsterer',
    'upholsterers',
    'upholstering',
    'upholsters',
    'upholstery',
    'upjohn',
    'upkeep',
    'upland',
    'uplands',
    'uplift',
    'uplifted',
    'uplifting',
    'uplink',
    'upload',
    'upmarket',
    'upmost',
    'upon',
    'upper',
    'uppercase',
    'uppercut',
    'uppermost',
    'uppers',
    'uppish',
    'uppishness',
    'uppity',
    'uppityness',
    'uppp',
    'uppsala',
    'upraise',
    'upraised',
    'upright',
    'uprightness',
    'uprights',
    'uprise',
    'uprising',
    'upriver',
    'uproar',
    'uproarious',
    'uproot',
    'uprooted',
    'uprooter',
    'uprooting',
    'uprose',
    'ups',
    'upsala',
    'upscale',
    'upset',
    'upsets',
    'upsetter',
    'upsetting',
    'upshot',
    'upside',
    'upsilon',
    'upstage',
    'upstaged',
    'upstager',
    'upstages',
    'upstair',
    'upstairs',
    'upstanding',
    'upstart',
    'upstream',
    'upstroke',
    'upsurge',
    'uptake',
    'upthrow',
    'upthrust',
    'uptick',
    'uptight',
    'uptime',
    'uptown',
    'upturn',
    'upturned',
    'upupa',
    'upupidae',
    'upward',
    'upwardly',
    'upwards',
    'upwind',
    'ur',
    'uracil',
    'uraemia',
    'uraemic',
    'ural',
    'uralic',
    'urals',
    'uranalysis',
    'urania',
    'uraninite',
    'uranium',
    'uranologist',
    'uranology',
    'uranoplasty',
    'uranoscopidae',
    'uranus',
    'uranyl',
    'urarthritis',
    'urate',
    'uratemia',
    'urates',
    'uraturia',
    'urban',
    'urbana',
    'urbane',
    'urbanisation',
    'urbanise',
    'urbanised',
    'urbanity',
    'urbanization',
    'urbanize',
    'urbanized',
    'urceolate',
    'urceole',
    'urchin',
    'urchins',
    'urd',
    'urdu',
    'urea',
    'urease',
    'uredinales',
    'uremia',
    'uremic',
    'ureter',
    'ureteritis',
    'ureterocele',
    'ureterostenosis',
    'ureters',
    'urethane',
    'urethra',
    'urethral',
    'urethritis',
    'urethrocele',
    'urex',
    'urey',
    'urga',
    'urge',
    'urged',
    'urgency',
    'urgent',
    'urgently',
    'urges',
    'urginea',
    'urging',
    'uria',
    'uriah',
    'urial',
    'uric',
    'uricaciduria',
    'uricosuric',
    'urinal',
    'urinalysis',
    'urinary',
    'urinate',
    'urinated',
    'urinates',
    'urination',
    'urinator',
    'urine',
    'url',
    'urmia',
    'urn',
    'urns',
    'urobilin',
    'urobilinogen',
    'urobilinogens',
    'urobilins',
    'urocele',
    'urochesia',
    'urochezia',
    'urochord',
    'urochorda',
    'urochordata',
    'urochordate',
    'urocyon',
    'urocystis',
    'urodele',
    'urodella',
    'urodynia',
    'urogenital',
    'urokinase',
    'urolith',
    'urologist',
    'urology',
    'uropathy',
    'urophycis',
    'uropsilus',
    'uropygi',
    'uropygium',
    'urosaurus',
    'ursa',
    'ursidae',
    'ursine',
    'ursinia',
    'ursus',
    'urth',
    'urtica',
    'urticaceae',
    'urticales',
    'urticaria',
    'urticate',
    'urtication',
    'urubupunga',
    'uruguay',
    'uruguayan',
    'urus',
    'us',
    'usa',
    'usability',
    'usable',
    'usableness',
    'usacil',
    'usaf',
    'usage',
    'usages',
    'usance',
    'usbeg',
    'usbek',
    'uscb',
    'usda',
    'use',
    'useable',
    'useableness',
    'used',
    'useful',
    'usefulness',
    'useless',
    'uselessness',
    'user',
    'users',
    'uses',
    'ushas',
    'usher',
    'ushered',
    'usherette',
    'ushering',
    'ushers',
    'using',
    'uskub',
    'usmc',
    'usn',
    'usnea',
    'usneaceae',
    'usps',
    'uss',
    'ussher',
    'ussr',
    'usss',
    'ustilaginaceae',
    'ustilaginales',
    'ustilaginoidea',
    'ustilago',
    'ustinov',
    'usual',
    'usually',
    'usualness',
    'usufruct',
    'usufructuary',
    'usuli',
    'usumbura',
    'usurer',
    'usurious',
    'usurp',
    'usurpation',
    'usurped',
    'usurper',
    'usurping',
    'usury',
    'ut',
    'uta',
    'utah',
    'utahan',
    'utahraptor',
    'utc',
    'ute',
    'utensil',
    'utensils',
    'uterine',
    'uterus',
    'utes',
    'utica',
    'utile',
    'utilisation',
    'utilise',
    'utilised',
    'utiliser',
    'utilitarian',
    'utilitarianism',
    'utilities',
    'utility',
    'utilizable',
    'utilization',
    'utilize',
    'utilized',
    'utilizer',
    'utilizers',
    'utilizing',
    'utmost',
    'utn',
    'utnapishtim',
    'utopia',
    'utopian',
    'utopianism',
    'utrecht',
    'utricle',
    'utricularia',
    'utriculus',
    'utrillo',
    'uttar',
    'utter',
    'utterable',
    'utterance',
    'utterances',
    'uttered',
    'utterer',
    'uttering',
    'utterly',
    'uttermost',
    'utterness',
    'utters',
    'utu',
    'utug',
    'uub',
    'uuh',
    'uup',
    'uuq',
    'uut',
    'uv',
    'uvea',
    'uveal',
    'uveitis',
    'uveoscleral',
    'uveous',
    'uvula',
    'uvular',
    'uvularia',
    'uvulariaceae',
    'uvulitis',
    'uvulopalatopharyngoplasty',
    'uxor',
    'uxorial',
    'uxoricide',
    'uxorious',
    'uxoriousness',
    'uygur',
    'uzbak',
    'uzbeg',
    'uzbek',
    'uzbekistan',
    'uzbekistani',
    'uzi']



from .wordnet_a import wordnet_terms_a
from .wordnet_b import wordnet_terms_b
from .wordnet_c import wordnet_terms_c
from .wordnet_d import wordnet_terms_d
from .wordnet_e import wordnet_terms_e
from .wordnet_f import wordnet_terms_f
from .wordnet_g import wordnet_terms_g
from .wordnet_h import wordnet_terms_h
from .wordnet_i import wordnet_terms_i
from .wordnet_j import wordnet_terms_j
from .wordnet_k import wordnet_terms_k
from .wordnet_l import wordnet_terms_l
from .wordnet_m import wordnet_terms_m
from .wordnet_n import wordnet_terms_n
from .wordnet_o import wordnet_terms_o
from .wordnet_p import wordnet_terms_p
from .wordnet_q import wordnet_terms_q
from .wordnet_r import wordnet_terms_r
from .wordnet_s import wordnet_terms_s
from .wordnet_t import wordnet_terms_t
from .wordnet_u import wordnet_terms_u
from .wordnet_v import wordnet_terms_v
from .wordnet_w import wordnet_terms_w
from .wordnet_x import wordnet_terms_x
from .wordnet_y import wordnet_terms_y
from .wordnet_z import wordnet_terms_z



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Apply Trailing Spaces to Text where appropriate """


from lingpatlab.baseblock import BaseObject


class TokenParserNormalize(BaseObject):
    """ Apply Trailing Spaces to Text where appropriate """

    def __init__(self):
        """ Change Log

        Created:
            13-Oct-2021
            craigtrim@gmail.com
            *   refactored out of 'parse-input-tokens' in pursuit of
                https://github.com/grafflr/graffl-core/issues/41
        Updated:
            14-Oct-2021
            craigtrim@gmail.com
            *   remove coordinate repositioning logic
                https://github.com/grafflr/graffl-core/issues/48#issuecomment-943793697
        Updated:
            16-Sept-2022
            craigtrim@gmail.com
            *   rename component
                https://github.com/craigtrim/spacy-token-parser/issues/3
        """
        BaseObject.__init__(self, __name__)

    @staticmethod
    def process(tokens: list) -> list:
        """Add Trailing Spaces to Text Attribute where appropriate

        This algorithm permits a function like this
            ''.join([x['text'] for x in results])
        to produce properly formatted output like this
            "every" time ("Test's") for now!

        where a naive join like this
            ' '.join([x['text'] for x in results])
        would create output like this
            " every " time ( " Test's " ) for now !

        Args:
            tokens (list): list of tokens

        Returns:
            list: list of tokens
        """
        results = []

        dquot_ctr = 0
        for i in range(len(tokens)):

            has_next_token = i + 1 < len(tokens)

            text = tokens[i]['text']

            # ---------------------------------------------------------- ##
            # Purpose:    Remove Coordinate Positioning Logic
            #             this has been placed into the upstream component 'graffl-parser-resultset'
            # Reference:  https://github.com/grafflr/graffl-core/issues/48#issuecomment-943793697
            # ---------------------------------------------------------- ##
            # def fmt_text() -> str:
            #     # ---------------------------------------------------------- ##
            #     # Purpose:    Improve Coordinate Positioning
            #     # Reference:  https://github.com/grafflr/graffl-core/issues/41#issuecomment-942808974
            #     # ---------------------------------------------------------- ##
            #     if text.strip() == '(':
            #         return text.strip()
            #     if not has_next_token:
            #         return text.strip()
            #     return f"{text} "

            tokens[i]['text'] = text

            if text == '"':
                dquot_ctr += 1
                if dquot_ctr % 2 == 1:
                    tokens[i]['text'] = text

            if has_next_token:
                if tokens[i + 1]['text'] in [')', '"', '!', '?']:
                    tokens[i]['text'] = tokens[i]['text'].strip()

            results.append(tokens[i])

        return results



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Transform unigram squots into unigram dquots """


from lingpatlab.baseblock import BaseObject


class TokenParserSquots(BaseObject):
    """ Transform unigram squots into unigram dquots """

    def __init__(self):
        """ Change Log

        Created:
            13-Oct-2021
            craigtrim@gmail.com
            *   refactored out of 'parse-input-tokens' in pursuit of
                https://github.com/grafflr/graffl-core/issues/41
        Updated:
            16-Sept-2022
            craigtrim@gmail.com
            *   rename component
                https://github.com/craigtrim/spacy-token-parser/issues/3
        """
        BaseObject.__init__(self, __name__)

    def process(self,
                tokens: list) -> list:
        """Transform unigram squots into unigram dquots

        Reference:
             https://github.com/grafflr/graffl-core/issues/1#issuecomment-934967879

        Args:
            tokens (list): list of tokens

        Returns:
            list: list of token results
        """
        results = []

        # ---------------------------------------------------------- ##
        # Note:         Eliminate Trailing Whitespace for accurate dependency parsing
        # Reference:    https://github.com/craigtrim/fast-sentence-tokenize/issues/4#issuecomment-1236220669
        # ---------------------------------------------------------- ##
        # tokens = [x.strip() for x in tokens]
        # tokens = [x for x in tokens if x and len(x)]

        for token in tokens:
            if token == "'":
                token = '"'
            results.append(token)

        return results



from typing import List

from lingpatlab.baseblock import Enforcer

from .bp import *
from .bp.tokenizer import Tokenizer
from .dmo import *
from .dto import *
from .svc import *

__tok = Tokenizer().input_text


def tokenize_text(input_text: str,
                  eliminate_whitespace: bool = True) -> list:
    """ Tokenize Input Text

    Args:
        input_text (str): any input string of any length
        eliminate_whitespace (bool, optional): strip whitespace from tokens. Defaults to True.

    Returns:
        list: a list of tokens
    """

    Enforcer.is_str(input_text)

    tokens = __tok(input_text)

    Enforcer.is_list_of_str(tokens)

    if eliminate_whitespace:
        # ----------------------------------------------------------
        # Note:         Eliminate Trailing Whitespace for accurate dependency parsing
        # Reference:    https://github.com/craigtrim/fast-sentence-tokenize/issues/4#issuecomment-1236220669
        # ----------------------------------------------------------
        tokens = [x.strip() for x in tokens]
        tokens = [x for x in tokens if x and len(x)]

    return tokens



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Graffl Tokenizer """


from typing import Optional, List
from lingpatlab.tokenizer.dmo import DictionaryFinder


class TokenizeUseGraffl(object):
    """ Graffl Tokenizer """

    __acronym_delimiter = '~~'

    def __init__(self):
        """ Change Log

        Created:
            29-Sept-2021
            craigtrim@gmail.com
        Updated:
            14-Oct-2021
            craigtrim@gmail.com
            *   added redelimit-spaces function
                https://github.com/grafflr/graffl-core/issues/48#issuecomment-943776908
        Updated:
            25-Aug-2022
            craigtrim@gmail.com
            *   keep underscored spaces together
                https://github.com/craigtrim/fast-sentence-tokenize/issues/1
            *   handle lingering squotes
                https://github.com/craigtrim/fast-sentence-tokenize/issues/2
        Updated:
            24-Jan-2024
            craigtrim@gmail.com
            *   avoid list index out of error exception
                https://github.com/craigtrim/modai/issues/3
        Updated:
            27-Feb-2024
            craigtrim@gmail.com
            *   migrate to spacy-core in pursuit of
                https://github.com/craigtrim/datapipe-apis/issues/43
        """
        pass

    def _replace_enclitics(self,
                           tokens: list) -> list:

        def transform(token: str) -> list:
            if "'" not in token:
                return [token]
            if token not in DictionaryFinder.enclitics():
                return [token]
            return DictionaryFinder.enclitics()[token]

        normalized = []
        for token in tokens:
            [normalized.append(x) for x in transform(token)]

        return normalized

    def _redelimit_acronyms(self,
                            tokens: list) -> list:

        def transform(token: str) -> str:
            if token.count('.') < 2:
                return token
            return token.replace('.', self.__acronym_delimiter)

        return [transform(x) for x in tokens]

    def _redelimit_abbreviations(self,
                                 tokens: list) -> list:

        def transform(token: str) -> str:
            if token not in DictionaryFinder.abbreviations():
                return token
            return DictionaryFinder.abbreviations()[token]

        return [transform(x) for x in tokens]

    def _undelimit_acronyms(self,
                            tokens: list) -> list:

        def transform(token: str) -> str:
            if self.__acronym_delimiter not in token:
                return token
            return token.replace(self.__acronym_delimiter, '.')

        return [transform(x) for x in tokens]

    def _handle_lingering_squotes(self,
                                  tokens: list) -> list:

        # -------------------------------------------------------------
        # Reference:    https://github.com/craigtrim/fast-sentence-tokenize/issues/2#issuecomment-1227902982
        # Purpose:      Handle first pattern
        # -------------------------------------------------------------
        normalized = []
        for token in tokens:

            if "'" not in token:
                normalized.append(token)

            # considered a valid suffix squote
            elif token.endswith("s' "):
                normalized.append(token)

            # considered an invalid suffix squot
            elif token.endswith("' "):
                normalized.append(f'{token[:-2]} ')
                normalized.append("'")

            else:
                normalized.append(token)

        tokens = normalized

        # -------------------------------------------------------------
        # Reference:    https://github.com/craigtrim/fast-sentence-tokenize/issues/2#issuecomment-1227905278
        # Purpose:      now find the more complex pattern
        # -------------------------------------------------------------
        squotes_pos = []
        squotes_suffix = []

        for i in range(len(tokens)):
            if tokens[i] == "'":
                squotes_pos.append(i)
            if len(tokens[i]) > 1 and tokens[i].strip().endswith("'"):
                squotes_suffix.append(i)

        # the issue does not exist
        if not squotes_suffix or not len(squotes_suffix):
            return tokens

        # -------------------------------------------------------------
        # Updated:          24-Jan-24
        # Reference:        https://github.com/craigtrim/modai/issues/3
        # Purpose:          Avoid List Index Out of Range Defect
        # -------------------------------------------------------------
        if not squotes_pos or not len(squotes_pos):
            return tokens

        # this implies there are an even-number of squotes, and thus,
        # it is likely that they are evenly balanced
        total_squotes = len(squotes_pos)
        if total_squotes > 0 and total_squotes % 2 != 1:
            return tokens

        # the standalone squote occurs after the suffix squote ...
        if squotes_pos[-1] > squotes_suffix[0]:
            return tokens

        normalized = []
        for i in range(len(tokens)):
            if i == squotes_suffix[0]:
                normalized.append(f'{tokens[i][:-1]} ')
                normalized.append("'")
            else:
                normalized.append(tokens[i])

        return normalized

    def _handle_punkt(self,
                      tokens: list) -> list:

        def transform(token: str) -> list:

            if token.isalpha():
                return [token]

            master = []
            buffer = []

            token_len = len(token)

            for i in range(token_len):

                def get_p1() -> Optional[str]:
                    if i - 1 >= 0:
                        return token[i - 1]

                def get_p2() -> Optional[str]:
                    if i - 2 >= 0:
                        return token[i - 2]

                def get_n1() -> Optional[str]:
                    if i + 1 < token_len:
                        return token[i + 1]

                def get_n2() -> Optional[str]:
                    if i + 2 < token_len:
                        return token[i + 2]

                p2 = get_p2()
                p1 = get_p1()
                ch = token[i]
                n1 = get_n1()
                n2 = get_n2()

                # fast-sentence-tokenize#1; keep underscored entities together
                if ch.isalpha() or ch.isnumeric() or ch in [' ', '_']:
                    buffer.append(ch)
                elif ch in ['.', ','] and p1 and p1.isnumeric():
                    buffer.append(ch)  # e.g., 1.06 or 325,000
                elif ch == "'" and p1 and p1.isalpha():
                    buffer.append(ch)  # e.g., Women's
                elif ch == '&' and p1 and p1.isalpha() and n1 and n1.isalpha():
                    buffer.append(ch)  # e.g., A&W
                else:
                    if len(buffer):
                        master.append(''.join(buffer))
                    if len(ch):
                        master.append(ch)
                    buffer = []

                i += 1

            if len(buffer):
                master.append(''.join(buffer))

            return master

        normalized = []
        for token in tokens:
            [normalized.append(x) for x in transform(token)]

        return normalized

    def _redelimit_spaces(self,
                          tokens: list) -> list:
        """Rejoin orphaned spaces with owning text

        Sample Input:
            ["health's ", 'management', ',', ' ', 'NPs']

        Sample Output
            ["health's ", 'management', ', ', 'NPs']

        Args:
            tokens (list): a list of tokens

        Returns:
            list: a normalized list of tokens
        """
        normalized = []

        i = 0
        while i < len(tokens):

            def next_token() -> Optional[str]:
                if i + 1 < len(tokens):
                    return tokens[i + 1]

            t_curr = tokens[i]
            t_next = next_token()

            if t_next and t_next == ' ':
                normalized.append(f'{t_curr} ')
                i += 1
            else:
                normalized.append(t_curr)

            i += 1

        return normalized

    def _split(self,
               input_text: str) -> list:
        master = []

        buffer = []
        for ch in input_text:
            buffer.append(ch)

            if ch in [' ']:
                master.append(''.join(buffer))
                buffer = []

        if len(buffer):
            master.append(''.join(buffer))

        return master

    def process(self,
                input_text: str) -> List[str]:

        tokens = self._split(input_text)
        tokens = self._replace_enclitics(tokens)
        tokens = self._redelimit_acronyms(tokens)
        tokens = self._redelimit_abbreviations(tokens)
        tokens = self._handle_punkt(tokens)
        tokens = self._redelimit_spaces(tokens)
        tokens = self._undelimit_acronyms(tokens)
        tokens = self._handle_lingering_squotes(tokens)

        return tokens



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Sentence Segmentation """


import spacy

from lingpatlab.baseblock import BaseObject
from lingpatlab.segmenter.dmo import (
    BulletPointCleaner,
    DelimitersToPeriods,
    NewlinesToPeriods,
    NumberedListNormalizer,
    PostProcessStructure,
    SpacyDocSegmenter
)


class PerformSentenceSegmentation(BaseObject):
    """ Sentence Segmentation """

    __nlp = None

    def __init__(self):
        """ Change Log

        Created:
            30-Sept-2021
            craigtrim@gmail.com
        Updated:
            19-Oct-2022
            craigtrim@gmail.com
            *   add numbered-list normalization
                https://github.com/craigtrim/fast-sentence-segment/issues/1
        Updated:
            27-Mar-2024
            craigtrim@gmail.com
            *   migrated out of modai in pursuit of
                https://github.com/craigtrim/datapipe-apis/issues/72
        """
        BaseObject.__init__(self, __name__)
        if not self.__nlp:
            self.__nlp = spacy.load("en_core_web_sm")

        self._delimiters_to_periods = DelimitersToPeriods.process
        self._newlines_to_periods = NewlinesToPeriods.process
        self._normalize_numbered_lists = NumberedListNormalizer().process
        self._clean_bullet_points = BulletPointCleaner.process
        self._spacy_segmenter = SpacyDocSegmenter(self.__nlp).process
        self._post_process = PostProcessStructure().process

    @staticmethod
    def _clean_punctuation(input_text: str) -> str:
        """ Purpose:
            Clean punctuation oddities; this is likely highly overfitted (for now)
        """
        if ", Inc" in input_text:
            input_text = input_text.replace(", Inc", " Inc")

        return input_text

    @staticmethod
    def _clean_spacing(a_sentence: str) -> str:

        # eliminate triple-space
        a_sentence = a_sentence.replace('   ', '  ')

        # treat double-space as delimiter
        a_sentence = a_sentence.replace('  ', '. ')

        return a_sentence

    def _process(self,
                 input_text: str) -> list:

        input_text = self._delimiters_to_periods(
            delimiter=',',
            input_text=input_text)

        input_text = self._delimiters_to_periods(
            delimiter=';',
            input_text=input_text)

        input_text = self._normalize_numbered_lists(input_text)

        input_text = self._newlines_to_periods(input_text)

        input_text = self._clean_spacing(input_text)
        if "." not in input_text:
            return [input_text]

        input_text = self._clean_bullet_points(input_text)
        if "." not in input_text:
            return [input_text]

        input_text = self._clean_punctuation(input_text)
        if "." not in input_text:
            return [input_text]

        sentences = self._spacy_segmenter(input_text)
        if "." not in input_text:
            return [input_text]

        sentences = self._post_process(sentences)

        sentences = [
            self._normalize_numbered_lists(x, denormalize=True)
            for x in sentences
        ]

        return sentences

    def process(self,
                input_text: str) -> list:
        """Perform Sentence Segmentation

        Args:
            input_text (str): An input string of any length or type

        Raises:
            ValueError: input must be a string

        Returns:
            list:   a list of sentences
                    each list item is an input string of any length, but is a semantic sentence
        """

        if input_text is None or not len(input_text):
            raise ValueError("Empty Input")

        if type(input_text) != str:
            self.logger.warning(f"Invalid Input Text: {input_text}")
            return []

        return self._process(input_text)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Add Punctuation Flag to Token """


from lingpatlab.baseblock import BaseObject


class TokenParserCoordinates(BaseObject):
    """ Add Punctuation Flag to Token """

    def __init__(self):
        """ Change Log

        Created:
            13-Oct-2021
            craigtrim@gmail.com
            *   refactored out of 'parse-input-tokens' in pursuit of
                https://github.com/grafflr/graffl-core/issues/41
        Updated:
            16-Sept-2022
            craigtrim@gmail.com
            *   rename component
                https://github.com/craigtrim/spacy-token-parser/issues/3
        Updated:
            4-Jun-2024
            craigtrim@gmail.com
            *   check last_y is None
                https://github.com/craigtrim/LingPatLab/issues/1
        """
        BaseObject.__init__(self, __name__)

    def process(self,
                tokens: list) -> list:
        """Add (X,Y) coordinates to each token
            to represent the position of this token in the original string

        Args:
            tokens (list): list of tokens

        Raises:
            ValueError: Invalid Coordinate Positions

        Returns:
            list: list of tokens
        """
        results = []

        pos = 0
        last_y = None

        for token in tokens:
            token['x'] = pos

            token['y'] = pos + len(token['text'].strip())
            pos += len(token['text'])

            results.append(token)
            last_y = token['y']

        # ---------------------------------------------------------- ##
        # Purpose:  Validate Coordinate Positioning
        # Issue:    https://github.com/grafflr/graffl-core/issues/41#issuecomment-942803328
        # ---------------------------------------------------------- ##
        actual_len = sum([len(x['text']) for x in tokens])
        
        # ---------------------------------------------------------- ##
        # Purpose:  Add Additional Check for last_y
        # Issue:    https://github.com/craigtrim/LingPatLab/issues/1
        # Updated:  4-Jun-2024
        # ---------------------------------------------------------- ##
        if last_y is not None and last_y > actual_len:
            self.logger.error('\n'.join([
                'Coordinate Exceed Actual Length',
                f'\tTokens: {tokens}',
                f'\tActual Length: {actual_len}',
                f'\tLast Coordinate: {last_y}']))
            raise ValueError

        return results
